/*
 * @(#)TelnetDeamon.java	1.00 3/3/01
 * Copyright (c) 2001 by Paul Regenhardt,  All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Paul Regenhardt ("Confidential Information"). You shall not 
 * disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Paul Regenhardt.
 */

import java.util.*;
import java.io.*;
import java.net.*;

/** This class is the abstract root for a Telnet Client/Server example
 * it provides the underpinnings, but it does not interpret any commands.
 * The dervied class is expected to interpret the commands (but it won't
 * need to do any socket manipulation).
 *
 * @author Paul Regenhardt
 * @version 1.0
 */


public abstract class TelnetDeamon extends Object
{
    public static Configuration config;
    protected Vector connections = new Vector();

	/**
	 * This method loads the configuration file.
	 */
	static public void start(String args[])
	{
	    if (args.length < 1)
	        config = new Configuration("config.dat");
	    else 
	        config = new Configuration(args[0]);
	}
	static public String get(String key) {return (String) config.get(key);}
	
	/**
	 * The constructor opens the socket server and listens 
	 * for incoming connection requests.  When it finds one,
	 * it creates a TelnetThread.
	 */
	public TelnetDeamon()
	{
	    
	    // Initialize log files (not implemented)
	    
        // Listen on a socket for an incoming connection, this is the
        // main program loop.  A thread is started for each connection
	    try {  
            ServerSocket s = new ServerSocket(config.getInt("ServerSocket"));
	        while (true) {
	            Socket incoming = s.accept();
	            TelnetThread t = new TelnetThread(this, incoming);
	            t.setTimeout(15);  // in minutes, needed to ensure dead connections time out

	            connections.addElement(t);
                t.setupLink();
                welcomeMessage(t);
                setprompt(t);
	        }
	    } catch (Exception e) {
	        System.out.println(e);}
	}
	
	/**
	 * This method closes a Telnet connection and stops the process.
	 */
	void closeConnection(TelnetThread t)
	{
	        connections.removeElement(t);
	        t.cleanup();
	        t.stop();
	}
	/**
	 * This method requests the telnet thread to display a
	 * welcome message, it's a nice method to overload to get
	 * a specific welcome message for you application.
	 */
    public void welcomeMessage(TelnetThread t)
    {
        t.transmit("\nWelcome to the Client-Server demo\n\n");
    }
        
	/**
	 * This method sets the prompt for telnet thread to display.
	 * it's a nice method to overload to set
	 * a specific application prompt.
	 */
    public void setprompt(TelnetThread t)
    {
        t.setPrompt("> ");
    }

	/**
	* This is the main processing loop, it is an abstract message
	* and must be present in your derived class
	*/
    public abstract void performAction(TelnetThread t, String s);
}
